@@ -18,20 +18,28 @@ test_that("class constructors work", {
     expect_output(print(alg), "vector")
     alg$release()
 
-    # args in a character string
+    # full path in a character string
     alg <- new(GDALAlg, "vector convert")
     alginfo <- alg$info()
     expect_false(alginfo$has_subalgorithms)
     expect_output(print(alg), "convert")
     alg$release()
 
-    # args in a character vector
+    # cmd path as a character vector
     alg <- new(GDALAlg, c("vector", "convert"))
     alginfo <- alg$info()
     expect_false(alginfo$has_subalgorithms)
     expect_output(print(alg), "convert")
     alg$release()
 
+    # args in a character vector
+    f <- system.file("extdata/storml_elev.tif", package="gdalraster")
+    args <- c("--format=text", "--no-md", f)
+    expect_no_error(alg <- new(GDALAlg, "raster info", args))
+    expect_true(alg$run())
+    expect_true(length(nchar(alg$output())) == 1 && nchar(alg$output()) > 100)
+    alg$release()
+
     # args in a list
     f <- system.file("extdata/ynp_fires_1984_2022.gpkg", package = "gdalraster")
     f_out = file.path(tempdir(), "ynp_fire_year.tif")
@@ -108,6 +116,9 @@ test_that("algorithm and argument properties are returned correctly", {
     expect_equal(arginfo$dataset_type_flags, "RASTER")
     expect_equal(arginfo$dataset_output_flags, "OBJECT")
 
+    expect_true(is.list(alg$getExplicitlySetArgs()))
+    expect_equal(length(alg$getExplicitlySetArgs()), 0)
+
     rm(alginfo)
     rm(arginfo)
 
@@ -131,6 +142,13 @@ test_that("algorithm and argument properties are returned correctly", {
     expect_equal(arginfo$dataset_type_flags, "RASTER")
     expect_equal(arginfo$dataset_output_flags, "OBJECT")
 
+    set_args <- alg$getExplicitlySetArgs()
+    expect_true(is.list(set_args))
+    expect_equal(length(set_args), 4)
+    expect_true(startsWith(set_args$input, "<raster dataset object:") ||
+                startsWith(set_args$input, "<list of dataset objects"))
+    expect_true(startsWith(set_args$output, "<raster dataset object:"))
+
     rm(alginfo)
     rm(arginfo)
 
@@ -192,15 +210,17 @@ test_that("GDALAlg S4 show() works", {
 test_that("algorithm run/output/outputs work", {
     f <- system.file("extdata/storml_elev.tif", package="gdalraster")
 
+    ## basic test with a few different arg types
+    f_clip <- file.path(tempdir(), "elev_clip.tif")
+
     args <- list()
     args$input <- f
-    f_clip <- file.path(tempdir(), "elev_clip.tif")
+    on.exit(deleteDataset(f_clip), add = TRUE)
     args$output <- f_clip
     args$overwrite <- TRUE
     args$bbox <- c(323776.1, 5102172.0,  327466.1, 5104782.0)
 
     expect_no_error(alg <- new(GDALAlg, "raster clip", args))
-
     expect_true(alg$run())
 
     # get the single output
@@ -219,7 +239,86 @@ test_that("algorithm run/output/outputs work", {
     expect_true(alg$close())
     alg$release()
 
-    deleteDataset(f_clip)
+    ## DATASET_LIST arg as vector of filenames and REAL_LIST arg
+    b4_file <- system.file("extdata/sr_b4_20200829.tif", package="gdalraster")
+    b5_file <- system.file("extdata/sr_b5_20200829.tif", package="gdalraster")
+    b6_file <- system.file("extdata/sr_b6_20200829.tif", package="gdalraster")
+    band_files <- c(b6_file, b5_file, b4_file)
+
+    f_vrt <- file.path(tempdir(), "stack_test.vrt")
+    on.exit(deleteDataset(f_vrt), add = TRUE)
+
+    args <- c("--dst-nodata=1,2,3", "--overwrite", band_files, f_vrt)
+    alg <- new(GDALAlg, "raster stack", args)
+    expect_true(alg$run())
+
+    ds_stack <- alg$output()
+    expect_true(is(ds_stack, "Rcpp_GDALRaster"))
+    expect_equal(ds_stack$dim(),  c(149, 112, 3))
+    expect_equal(ds_stack$getNoDataValue(band = 1), 1)
+    expect_equal(ds_stack$getNoDataValue(band = 2), 2)
+    expect_equal(ds_stack$getNoDataValue(band = 3), 3)
+
+    ds_stack$close()
+    expect_true(alg$close())
+    alg$release()
+
+    ## DATASET_LIST arg as objects and REAL_LIST as numeric vector
+    ds_list <- list(new(GDALRaster, b6_file),
+                    new(GDALRaster, b5_file),
+                    new(GDALRaster, b4_file))
+
+    f_vrt2 <- file.path(tempdir(), "stack_test2.vrt")
+    on.exit(deleteDataset(f_vrt2), add = TRUE)
+
+    args <- list("input" = ds_list,
+                 "output" = f_vrt2,
+                 "overwrite" = TRUE,
+                 "dst_nodata" = c(1, 2, 3))
+
+    alg <- new(GDALAlg, "raster stack", args)
+    expect_true(alg$run())
+
+    ds_stack <- alg$output()
+    expect_true(is(ds_stack, "Rcpp_GDALRaster"))
+    expect_equal(ds_stack$dim(),  c(149, 112, 3))
+    expect_equal(ds_stack$getNoDataValue(band = 1), 1)
+    expect_equal(ds_stack$getNoDataValue(band = 2), 2)
+    expect_equal(ds_stack$getNoDataValue(band = 3), 3)
+
+    ds_stack$close()
+    for (i in seq_along(ds_list))
+        ds_list[[i]]$close()
+    alg$release()
+
+    ## STRING_LIST arg (--src-nodata and --dst-nodata are STRING_LIST in
+    ## "raster reproject")
+    args <- c("--output", "", "--format", "MEM", "--size=2,1", "--band-count=2",
+              "--datatype=Float32")
+    alg <- gdal_run("raster create", args)
+    ds <- alg$output()
+    alg$release()
+    expect_true(ds$setGeoTransform(c(0, 1, 0, 0, 0, -1)))
+    ds$write(1, 0, 0, 2, 1, c(-999, 1))
+    ds$write(2, 0, 0, 2, 1, c(NaN, 2))
+
+    args <- list("input" = ds,
+                 "output" = "",
+                 "format" = "MEM",
+                 "src-nodata" = c("-999", "nan"),
+                 "dst-nodata" = c("0", "0"))
+    alg <- gdal_run("raster reproject", args)
+    out_ds <- alg$output()
+    alg$release()
+    expect_equal(out_ds$read(1, 0, 0, 2, 1, 2, 1), c(NA_real_, 1))
+    expect_equal(out_ds$read(2, 0, 0, 2, 1, 2, 1), c(NA_real_, 2))
+    out_ds$deleteNoDataValue(band = 1)
+    out_ds$deleteNoDataValue(band = 2)
+    expect_equal(out_ds$read(1, 0, 0, 2, 1, 2, 1), c(0, 1))
+    expect_equal(out_ds$read(2, 0, 0, 2, 1, 2, 1), c(0, 2))
+
+    out_ds$close()
+    ds$close()
 })
 
 test_that("`setVectorArgsFromObject` and `outputLayerNameForOpen` work", {
@@ -322,7 +421,8 @@ test_that("`setVectorArgsFromObject` and `outputLayerNameForOpen` work", {
 
     ## auto set "sql" with "dialect"
     shp_dsn <- system.file("extdata/poly_multipoly.shp", package = "gdalraster")
-    sql <- "SELECT rowid AS fid, ST_Centroid(geometry) As geom FROM poly_multipoly"
+    sql <- "SELECT rowid AS fid, ST_Centroid(geometry) As geom
+            FROM poly_multipoly"
     if (has_spatialite()) {
         lyr_in <- new(GDALVector, shp_dsn, sql, TRUE, NULL, "", "SQLite")
 
@@ -474,3 +574,177 @@ test_that("`setVectorArgsFromObject` and `outputLayerNameForOpen` work", {
     ds_out$close()
     lyr$close()
 })
+
+test_that("setArg works", {
+    ## set args individually by API
+    f_dem <- system.file("extdata/storml_elev.tif", package="gdalraster")
+    ds <- new(GDALRaster, f_dem)
+
+    f_aspect <- file.path(tempdir(), "aspect_test.tif")
+    on.exit(deleteDataset(f_aspect), add = TRUE)
+
+    alg <- new(GDALAlg, "raster aspect")
+    expect_true(alg$setArg("input", ds))
+    expect_true(alg$setArg("output", f_aspect))
+    expect_true(alg$setArg("creation-option", "COMPRESS=DEFLATE"))
+    expect_true(alg$setArg("overwrite", TRUE))
+    expect_true(alg$setArg("zero-for-flat", TRUE))
+    expect_true(alg$run())
+
+    ds_aspect <- alg$output()
+    expect_true(is(ds_aspect, "Rcpp_GDALRaster"))
+    minmax <- ds_aspect$getMinMax(band = 1, approx_ok = FALSE)
+    expect_equal(minmax[1], 0)
+    expect_true(minmax[2] >= 359 && minmax[2] <= 360)
+    compression <- ds_aspect$getMetadataItem(band = 0, "COMPRESSION",
+                                             "IMAGE_STRUCTURE")
+    expect_equal(compression, "DEFLATE")
+    ds_aspect$close()
+    expect_true(alg$close())
+    alg$release()
+
+    ## instantiate with args, then set additional args individually
+    f_slpp <- file.path(tempdir(), "slpp_test.tif")
+    on.exit(deleteDataset(f_slpp), add = TRUE)
+
+    args <- list()
+    args$input <- ds
+    args$output <- f_slpp
+    args$creation_option <- "COMPRESS=DEFLATE"
+    args$overwrite <- TRUE
+    alg <- new(GDALAlg, "raster slope", args)
+    expect_true(alg$setArg("unit", "percent"))
+    expect_true(alg$run())
+
+    ds_slpp <- alg$output()
+    expect_true(is(ds_slpp, "Rcpp_GDALRaster"))
+    minmax <- ds_slpp$getMinMax(band = 1, approx_ok = FALSE)
+    expect_equal(minmax[1], 0)
+    expect_true(minmax[2] > 100 && minmax[2] < 200)
+    compression <- ds_slpp$getMetadataItem(band = 0, "COMPRESSION",
+                                           "IMAGE_STRUCTURE")
+    expect_equal(compression, "DEFLATE")
+    ds_slpp$close()
+    expect_true(alg$close())
+    alg$release()
+
+    ## instantiate with args, then set additional args individually after
+    ## parsing command-line arguments (which also instantiates the actual
+    ## algorithm internally)
+    f_clip <- file.path(tempdir(), "clip_test.tif")
+    on.exit(deleteDataset(f_clip), add = TRUE)
+
+    args <- list()
+    args$input <- ds
+    args$output <- f_clip
+    args$overwrite <- TRUE
+    alg <- new(GDALAlg, "raster clip", args)
+    expect_true(alg$parseCommandLineArgs())
+    expect_true(
+        alg$setArg("bbox", c(323776.1, 5102172.0,  327466.1, 5104782.0)))
+    expect_true(alg$setArg("creation-option", "COMPRESS=DEFLATE"))
+    set_args <- alg$getExplicitlySetArgs()
+    expect_equal(set_args$bbox, c(323776.1, 5102172.0,  327466.1, 5104782.0),
+                 tolerance = 1e-5)
+    expect_equal(set_args$creation_option, "COMPRESS=DEFLATE")
+    expect_true(alg$run())
+
+    ds_clip <- alg$output()
+    expect_true(is(ds_clip, "Rcpp_GDALRaster"))
+    expect_equal(ds_clip$dim(), c(123, 87, 1))
+    compression <- ds_clip$getMetadataItem(band = 0, "COMPRESSION",
+                                           "IMAGE_STRUCTURE")
+    expect_equal(compression, "DEFLATE")
+    ds_clip$close()
+    expect_true(alg$close())
+    alg$release()
+
+    ds$close()
+
+    ## DATASET_LIST arg as charcter vector and REAL_LIST as numeric vector
+    b4_file <- system.file("extdata/sr_b4_20200829.tif", package="gdalraster")
+    b5_file <- system.file("extdata/sr_b5_20200829.tif", package="gdalraster")
+    b6_file <- system.file("extdata/sr_b6_20200829.tif", package="gdalraster")
+    band_files <- c(b6_file, b5_file, b4_file)
+
+    f_vrt <- file.path(tempdir(), "stack_test.vrt")
+    on.exit(deleteDataset(f_vrt), add = TRUE)
+
+    alg <- new(GDALAlg, "raster stack")
+    expect_true(alg$setArg("input", band_files))
+    expect_true(alg$setArg("output", f_vrt))
+    expect_true(alg$setArg("overwrite", TRUE))
+    expect_true(alg$setArg("dst-nodata", c(1, 2, 3)))
+    expect_true(alg$run())
+
+    ds_stack <- alg$output()
+    expect_true(is(ds_stack, "Rcpp_GDALRaster"))
+    expect_equal(ds_stack$dim(),  c(149, 112, 3))
+    expect_equal(ds_stack$getNoDataValue(band = 1), 1)
+    expect_equal(ds_stack$getNoDataValue(band = 2), 2)
+    expect_equal(ds_stack$getNoDataValue(band = 3), 3)
+
+    ds_stack$close()
+    alg$release()
+
+    ## DATASET_LIST arg as list of objects
+    ds_list <- list(new(GDALRaster, b6_file),
+                    new(GDALRaster, b5_file),
+                    new(GDALRaster, b4_file))
+
+    f_vrt2 <- file.path(tempdir(), "stack_test2.vrt")
+    on.exit(deleteDataset(f_vrt2), add = TRUE)
+
+    alg <- new(GDALAlg, "raster stack")
+    expect_true(alg$setArg("input", ds_list))
+    expect_true(alg$setArg("output", f_vrt2))
+    expect_true(alg$setArg("overwrite", TRUE))
+    expect_true(alg$run())
+
+    ds_stack <- alg$output()
+    expect_true(is(ds_stack, "Rcpp_GDALRaster"))
+    expect_equal(ds_stack$dim(),  c(149, 112, 3))
+    expect_equal(ds_stack$getNoDataValue(band = 1), 0)
+    expect_equal(ds_stack$getNoDataValue(band = 2), 0)
+    expect_equal(ds_stack$getNoDataValue(band = 3), 0)
+
+    ds_stack$close()
+    for (i in seq_along(ds_list))
+        ds_list[[i]]$close()
+    alg$release()
+
+    ## INTEGER_LIST (--size in "raster create")
+    ## STRING_LIST (--src-nodata and --dst-nodata in "raster reproject")
+    args <- c("--output", "", "--format", "MEM")
+    alg <- gdal_alg("raster create", args)
+    expect_true(alg$setArg("size", c(2, 1)))
+    expect_true(alg$setArg("band-count", 2))
+    expect_true(alg$setArg("datatype", "Float32"))
+    expect_true(alg$run())
+
+    ds <- alg$output()
+    alg$release()
+    expect_true(ds$setGeoTransform(c(0, 1, 0, 0, 0, -1)))
+    ds$write(1, 0, 0, 2, 1, c(-999, 1))
+    ds$write(2, 0, 0, 2, 1, c(NaN, 2))
+
+    args <- list("input" = ds,
+                 "output" = "",
+                 "format" = "MEM")
+    alg <- gdal_alg("raster reproject", args)
+    expect_true(alg$setArg("src-nodata", c("-999", "nan")))
+    expect_true(alg$setArg("dst-nodata", c("0", "0")))
+    expect_true(alg$run())
+
+    out_ds <- alg$output()
+    alg$release()
+    expect_equal(out_ds$read(1, 0, 0, 2, 1, 2, 1), c(NA_real_, 1))
+    expect_equal(out_ds$read(2, 0, 0, 2, 1, 2, 1), c(NA_real_, 2))
+    out_ds$deleteNoDataValue(band = 1)
+    out_ds$deleteNoDataValue(band = 2)
+    expect_equal(out_ds$read(1, 0, 0, 2, 1, 2, 1), c(0, 1))
+    expect_equal(out_ds$read(2, 0, 0, 2, 1, 2, 1), c(0, 2))
+
+    out_ds$close()
+    ds$close()
+})
